#include <ESP32-HUB75-MatrixPanel-I2S-DMA.h>

#define PANEL_RES_X 64 // 屏幕宽度
#define PANEL_RES_Y 64 // 屏幕高度
#define PANEL_CHAIN 1  // 单块屏幕

MatrixPanel_I2S_DMA *dma_display = nullptr;

// 包含 RGB565 格式的图片数据
#include "image.h"

// 函数：生成流动的彩虹颜色
uint16_t getRainbowColor(int x, int y, int timeOffset) {
  // 使用 x, y 和时间参数生成动态的彩虹颜色
  uint8_t red = (sin(x * 0.1 + timeOffset * 0.1) + 1) * 127;   // 红色通道
  uint8_t green = (sin(y * 0.1 + timeOffset * 0.1 + 2) + 1) * 127; // 绿色通道
  uint8_t blue = (sin(x * 0.1 + timeOffset * 0.1 + 4) + 1) * 127;  // 蓝色通道
  return dma_display->color565(red, green, blue);
}

void setup() {
  // 配置 LED 矩阵的引脚
  HUB75_I2S_CFG::i2s_pins _pins = {
    25, 26, 27, 14, 12, 13, 23, 19, 5, 17, 18, 4, 15, 16,
  };

  // 初始化屏幕配置
  HUB75_I2S_CFG mxconfig(PANEL_RES_X, PANEL_RES_Y, PANEL_CHAIN, _pins);

  // 初始化 DMA 显示对象
  dma_display = new MatrixPanel_I2S_DMA(mxconfig);
  dma_display->begin();
  dma_display->clearScreen();
  dma_display->setBrightness(255); // 设置亮度 (0-255)
}

void loop() {
  // 获取当前时间（用于动画效果）
  int timeOffset = millis() / 20; // 调整速度（50越小，速度越快）

  // 遍历图像数据
  for (int y = 0; y < PANEL_RES_Y; y++) {
    for (int x = 0; x < PANEL_RES_X; x++) {
      uint16_t color = image_data[y * PANEL_RES_X + x]; // 获取像素颜色

      // 检查是否是红色 (RGB565: 红色高位 5bit)
      if ((color & 0xF800) == 0xF800) { // 如果是红色
        color = getRainbowColor(x, y, timeOffset); // 替换为流动的彩虹颜色
      }

      // 绘制像素
      dma_display->drawPixel(x, y, color);
    }
  }
}